"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MethodMeta = exports.Role = void 0;
const path_1 = require("path");
var Role;
(function (Role) {
    Role["Admin"] = "ADMIN";
    Role["Owner"] = "OWNER";
    Role["Member"] = "MEMBER";
    Role["Public"] = "PUBLIC";
})(Role = exports.Role || (exports.Role = {}));
class MethodMeta {
    name = '';
    path = '';
    action = '';
    ref = null;
    role = null;
    params = [];
    constructor(name) {
        this.name = name;
    }
    bindApi(api, basePath, proto) {
        const method = this.ref;
        if (method == null) {
            throw new Error(`${this.name} method missing route decorator`);
        }
        if (this.role == null) {
            throw new Error(`${this.name} method missing role decorator`);
        }
        const path = '/' + (0, path_1.join)(basePath, this.path).replace(/^\//, '');
        const types = Reflect.getMetadata("design:paramtypes", proto, this.name);
        const params = types.map((type, i) => {
            const decoratorParams = this.params[i];
            if (!decoratorParams) {
                throw new Error(`${this.name} method missing param decorator at index ${i}`);
            }
            return { ...decoratorParams, type };
        });
        api[this.action](path, (req, res) => {
            return method.apply(proto, params.map(param => {
                let value;
                if (param.getter) {
                    value = param.getter(param.data, req, res);
                }
                if (param.validator) {
                    value = param.validator.parse(value);
                }
                else if (typeof param.type === 'function') {
                    value = param.type(value);
                }
                return value;
            }));
        });
        return {
            action: this.action.toUpperCase(),
            path: path,
            method: this.name,
            role: this.role
        };
    }
}
exports.MethodMeta = MethodMeta;
